@@ -1,24 +1,24 @@
-'''
+"""
 Contains base classes for modbus request/response/error packets
-'''
+"""
 from pymodbus.interfaces import Singleton
 from pymodbus.exceptions import NotImplementedException
 from pymodbus.constants import Defaults
 from pymodbus.utilities import rtuFrameSize
 from pymodbus.compat import iteritems, int2byte, byte2int
 
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 # Logging
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 import logging
 _logger = logging.getLogger(__name__)
 
 
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 # Base PDU's
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 class ModbusPDU(object):
-    '''
+    """
     Base class for all Modbus mesages
 
     .. attribute:: transaction_id
@@ -50,10 +50,10 @@ class ModbusPDU(object):
        to create a complicated message. By setting this to True, the
        request will pass the currently encoded message through instead
        of encoding it again.
-    '''
+    """
 
     def __init__(self, **kwargs):
-        ''' Initializes the base data for a modbus request '''
+        """ Initializes the base data for a modbus request """
         self.transaction_id = kwargs.get('transaction', Defaults.TransactionId)
         self.protocol_id = kwargs.get('protocol', Defaults.ProtocolId)
         self.unit_id = kwargs.get('unit', Defaults.UnitId)
@@ -61,27 +61,27 @@ class ModbusPDU(object):
         self.check = 0x0000
 
     def encode(self):
-        ''' Encodes the message
+        """ Encodes the message
 
         :raises: A not implemented exception
-        '''
+        """
         raise NotImplementedException()
 
     def decode(self, data):
-        ''' Decodes data part of the message.
+        """ Decodes data part of the message.
 
         :param data: is a string object
         :raises: A not implemented exception
-        '''
+        """
         raise NotImplementedException()
 
     @classmethod
     def calculateRtuFrameSize(cls, buffer):
-        ''' Calculates the size of a PDU.
+        """ Calculates the size of a PDU.
 
         :param buffer: A buffer containing the data that have been received.
         :returns: The number of bytes in the PDU.
-        '''
+        """
         if hasattr(cls, '_rtu_frame_size'):
             return cls._rtu_frame_size
         elif hasattr(cls, '_rtu_byte_count_pos'):
@@ -91,25 +91,25 @@ class ModbusPDU(object):
 
 
 class ModbusRequest(ModbusPDU):
-    ''' Base class for a modbus request PDU '''
+    """ Base class for a modbus request PDU """
 
     def __init__(self, **kwargs):
-        ''' Proxy to the lower level initializer '''
+        """ Proxy to the lower level initializer """
         ModbusPDU.__init__(self, **kwargs)
 
     def doException(self, exception):
-        ''' Builds an error response based on the function
+        """ Builds an error response based on the function
 
         :param exception: The exception to return
         :raises: An exception response
-        '''
+        """
         _logger.error("Exception Response F(%d) E(%d)" %
                 (self.function_code, exception))
         return ExceptionResponse(self.function_code, exception)
 
 
 class ModbusResponse(ModbusPDU):
-    ''' Base class for a modbus response PDU
+    """ Base class for a modbus response PDU
 
     .. attribute:: should_respond
 
@@ -120,22 +120,26 @@ class ModbusResponse(ModbusPDU):
 
        Indicates the size of the modbus rtu response used for
        calculating how much to read.
-    '''
+    """
 
     should_respond = True
 
     def __init__(self, **kwargs):
-        ''' Proxy to the lower level initializer '''
+        """ Proxy to the lower level initializer """
         ModbusPDU.__init__(self, **kwargs)
 
+    def isError(self):
+        """Checks if the error is a success or failure"""
+        return self.function_code > 0x80
 
-#---------------------------------------------------------------------------#
+
+# --------------------------------------------------------------------------- #
 # Exception PDU's
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 class ModbusExceptions(Singleton):
-    '''
+    """
     An enumeration of the valid modbus exceptions
-    '''
+    """
     IllegalFunction         = 0x01
     IllegalAddress          = 0x02
     IllegalValue            = 0x03
@@ -148,93 +152,96 @@ class ModbusExceptions(Singleton):
 
     @classmethod
     def decode(cls, code):
-        ''' Given an error code, translate it to a
-        string error name. 
-        
+        """ Given an error code, translate it to a
+        string error name.
+
         :param code: The code number to translate
-        '''
+        """
         values = dict((v, k) for k, v in iteritems(cls.__dict__)
             if not k.startswith('__') and not callable(v))
         return values.get(code, None)
 
 
 class ExceptionResponse(ModbusResponse):
-    ''' Base class for a modbus exception PDU '''
+    """ Base class for a modbus exception PDU """
     ExceptionOffset = 0x80
     _rtu_frame_size = 5
 
     def __init__(self, function_code, exception_code=None, **kwargs):
-        ''' Initializes the modbus exception response
+        """ Initializes the modbus exception response
 
         :param function_code: The function to build an exception response for
         :param exception_code: The specific modbus exception to return
-        '''
+        """
         ModbusResponse.__init__(self, **kwargs)
         self.original_code = function_code
         self.function_code = function_code | self.ExceptionOffset
         self.exception_code = exception_code
 
     def encode(self):
-        ''' Encodes a modbus exception response
+        """ Encodes a modbus exception response
 
         :returns: The encoded exception packet
-        '''
+        """
         return int2byte(self.exception_code)
 
     def decode(self, data):
-        ''' Decodes a modbus exception response
+        """ Decodes a modbus exception response
 
         :param data: The packet data to decode
-        '''
+        """
         self.exception_code = byte2int(data[0])
 
     def __str__(self):
-        ''' Builds a representation of an exception response
+        """ Builds a representation of an exception response
 
         :returns: The string representation of an exception response
-        '''
+        """
         message = ModbusExceptions.decode(self.exception_code)
         parameters = (self.function_code, self.original_code, message)
         return "Exception Response(%d, %d, %s)" % parameters
 
 
 class IllegalFunctionRequest(ModbusRequest):
-    '''
+    """
     Defines the Modbus slave exception type 'Illegal Function'
     This exception code is returned if the slave::
 
         - does not implement the function code **or**
         - is not in a state that allows it to process the function
-    '''
+    """
     ErrorCode = 1
 
     def __init__(self, function_code, **kwargs):
-        ''' Initializes a IllegalFunctionRequest
+        """ Initializes a IllegalFunctionRequest
 
         :param function_code: The function we are erroring on
-        '''
+        """
         ModbusRequest.__init__(self, **kwargs)
         self.function_code = function_code
 
     def decode(self, data):
-        ''' This is here so this failure will run correctly
+        """ This is here so this failure will run correctly
 
         :param data: Not used
-        '''
+        """
         pass
 
     def execute(self, context):
-        ''' Builds an illegal function request error response
+        """ Builds an illegal function request error response
 
         :param context: The current context for the message
         :returns: The error response packet
-        '''
+        """
         return ExceptionResponse(self.function_code, self.ErrorCode)
 
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 # Exported symbols
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
+
+
 __all__ = [
     'ModbusRequest', 'ModbusResponse', 'ModbusExceptions',
     'ExceptionResponse', 'IllegalFunctionRequest',
 ]
+
