@@ -17,6 +17,7 @@ from pymodbus.server.sync import ModbusTcpServer, ModbusUdpServer, ModbusSerialS
 from pymodbus.server.sync import StartTcpServer, StartUdpServer, StartSerialServer
 from pymodbus.exceptions import NotImplementedException
 from pymodbus.bit_read_message import ReadCoilsRequest, ReadCoilsResponse
+from pymodbus.datastore import  ModbusServerContext
 import sys
 from pymodbus.compat import socketserver
 
@@ -28,7 +29,7 @@ if sys.platform == "darwin":
 #---------------------------------------------------------------------------#
 class MockServer(object):
     def __init__(self):
-        self.framer  = lambda _: "framer"
+        self.framer  = lambda _, client=None: "framer"
         self.decoder = "decoder"
         self.threads = []
         self.context = {}
@@ -58,7 +59,6 @@ class SynchronousServerTest(unittest.TestCase):
         request = ReadCoilsRequest(1, 1)
         address = ('server', 12345)
         server  = MockServer()
-        
         with patch.object(ModbusBaseRequestHandler, 'handle') as mock_handle:
             with patch.object(ModbusBaseRequestHandler, 'send') as mock_send:
                 mock_handle.return_value = True
@@ -97,6 +97,8 @@ class SynchronousServerTest(unittest.TestCase):
         handler.framer  = Mock()
         handler.framer.buildPacket.return_value = b"message"
         handler.request = Mock()
+        handler.socket = Mock()
+        handler.server = Mock()
         handler.request.recv.return_value = b"\x12\x34"
 
         # exit if we are not running
@@ -105,15 +107,18 @@ class SynchronousServerTest(unittest.TestCase):
         self.assertEqual(handler.framer.processIncomingPacket.call_count, 0)
 
         # run forever if we are running
-        def _callback1(a, b):
+        def _callback1(a, b, *args, **kwargs):
             handler.running = False # stop infinite loop
         handler.framer.processIncomingPacket.side_effect = _callback1
         handler.running = True
+        # Ugly hack
+        handler.server.context = ModbusServerContext(slaves={-1: None},
+                                                     single=False)
         handler.handle()
         self.assertEqual(handler.framer.processIncomingPacket.call_count, 1)
 
         # exceptions are simply ignored
-        def _callback2(a, b):
+        def _callback2(a, b, *args, **kwargs):
             if handler.framer.processIncomingPacket.call_count == 2:
                 raise Exception("example exception")
             else: handler.running = False # stop infinite loop
@@ -128,7 +133,7 @@ class SynchronousServerTest(unittest.TestCase):
     def testModbusConnectedRequestHandlerSend(self):
         handler = socketserver.BaseRequestHandler(None, None, None)
         handler.__class__ = ModbusConnectedRequestHandler
-        handler.framer  = Mock()
+        handler.framer = Mock()
         handler.framer.buildPacket.return_value = b"message"
         handler.request = Mock()
         request = ReadCoilsResponse([1])
@@ -142,6 +147,9 @@ class SynchronousServerTest(unittest.TestCase):
     def testModbusConnectedRequestHandlerHandle(self):
         handler = socketserver.BaseRequestHandler(None, None, None)
         handler.__class__ = ModbusConnectedRequestHandler
+        handler.server = Mock()
+        # handler.server.context.slaves = Mock()
+        # protocol.factory.store.single = True
         handler.framer  = Mock()
         handler.framer.buildPacket.return_value = b"message"
         handler.request = Mock()
@@ -153,7 +161,7 @@ class SynchronousServerTest(unittest.TestCase):
         self.assertEqual(handler.framer.processIncomingPacket.call_count, 0)
 
         # run forever if we are running
-        def _callback(a, b):
+        def _callback(a, b, *args, **kwargs):
             handler.running = False # stop infinite loop
         handler.framer.processIncomingPacket.side_effect = _callback
         handler.running = True
@@ -176,7 +184,7 @@ class SynchronousServerTest(unittest.TestCase):
         handler.request.recv.return_value = None
         handler.running = True
         handler.handle()
-        self.assertEqual(handler.framer.processIncomingPacket.call_count, 3)
+        self.assertEqual(handler.framer.processIncomingPacket.call_count, 4)
 
     #-----------------------------------------------------------------------#
     # Test Disconnected Request Handler
@@ -185,20 +193,23 @@ class SynchronousServerTest(unittest.TestCase):
         handler = socketserver.BaseRequestHandler(None, None, None)
         handler.__class__ = ModbusDisconnectedRequestHandler
         handler.framer  = Mock()
+        handler.server = Mock()
         handler.framer.buildPacket.return_value = b"message"
         handler.request = Mock()
+        handler.socket = Mock()
         request = ReadCoilsResponse([1])
         handler.send(request)
-        self.assertEqual(handler.request.sendto.call_count, 1)
+        self.assertEqual(handler.socket.sendto.call_count, 1)
 
         request.should_respond = False
         handler.send(request)
-        self.assertEqual(handler.request.sendto.call_count, 1)
+        self.assertEqual(handler.socket.sendto.call_count, 1)
 
     def testModbusDisconnectedRequestHandlerHandle(self):
         handler = socketserver.BaseRequestHandler(None, None, None)
         handler.__class__ = ModbusDisconnectedRequestHandler
         handler.framer  = Mock()
+        handler.server = Mock()
         handler.framer.buildPacket.return_value = b"message"
         handler.request = (b"\x12\x34", handler.request)
 
@@ -233,7 +244,7 @@ class SynchronousServerTest(unittest.TestCase):
         handler.request = (None, handler.request)
         handler.running = True
         handler.handle()
-        self.assertEqual(handler.framer.processIncomingPacket.call_count, 3)
+        self.assertEqual(handler.framer.processIncomingPacket.call_count, 4)
 
     #-----------------------------------------------------------------------#
     # Test TCP Server
